package cd.casic.ci.process.process.service.sast.impl;

import cd.casic.ci.process.dto.req.sast.SastIdentifiedLanguageTaskReq;
import cd.casic.ci.process.dto.resp.sast.*;
import cd.casic.ci.process.process.service.sast.SastService;
import cd.casic.ci.process.properties.SastProperties;
import cd.casic.framework.redis.core.RedisTemplateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import jakarta.annotation.Resource;
import kotlin.text.Charsets;
import org.apache.commons.lang3.StringUtils;
import org.luaj.vm2.ast.Str;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.io.File;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cd.casic.ci.process.constant.SastUrlConstant.*;

@Service
public class SastServiceImpl implements SastService {
    @Resource
    private RedisTemplateUtils redisTemplateUtils;
    @Resource
    private SastProperties sastProperties;
    @Resource
    private RestTemplate restTemplate;
    public static final String TOKEN_PREFIX = "Bearer ";
    public static final String TOKEN_HEADER_KEY = "authorization";
    public static final String REDIS_SAST_TOKEN_KEY = "REDIS_SAST_TOKEN_KEY";

    private SastTokenResp getTokenRemote(){
        HttpHeaders httpHeaders = new HttpHeaders();
        HttpEntity<SastProperties> httpEntity = new HttpEntity<SastProperties>(sastProperties,httpHeaders);
        ResponseEntity<String> exchange = restTemplate.exchange(baseUrl+getToken, HttpMethod.POST, httpEntity, String.class, new HashMap<>());
        String body = exchange.getBody();
        JSONObject bodyObject = JSON.parseObject(body);
        SastTokenResp tokenResp = new SastTokenResp();
        tokenResp.setAccessToken(bodyObject.getString("access_token"));
        tokenResp.setExpiresIn(bodyObject.getInteger("expires_in"));
        tokenResp.setRefreshExpiresIn(bodyObject.getInteger("refresh_expires_in"));
        tokenResp.setRefreshToken(bodyObject.getString("refresh_token"));
        tokenResp.setTokenType(bodyObject.getString("token_type"));
        return tokenResp;
    }

    @Override
    public String getToken() {
        Object o = redisTemplateUtils.get(REDIS_SAST_TOKEN_KEY);
        String token = o instanceof String ? ((String) o) : "";
        if (StringUtils.isEmpty(token)) {
            synchronized(this){
                o = redisTemplateUtils.get(REDIS_SAST_TOKEN_KEY);
                token = o instanceof String ? ((String) o) : "";
                if (StringUtils.isEmpty(token)) {
                    SastTokenResp tokenRemote = getTokenRemote();
                    String accessToken = tokenRemote.getAccessToken();
                    redisTemplateUtils.set(REDIS_SAST_TOKEN_KEY,accessToken,tokenRemote.getExpiresIn()*1000);
                    token = accessToken;
                }
            }
        }
        return token;
    }

    @Override
    public Boolean applicationExist(String applicationName) {
        HttpHeaders headers = getHeaders();
        HttpEntity<Boolean> entity = new HttpEntity<>(null,headers);
        Map<String,Object> httpParams = new HashMap<>();
        String uriString = UriComponentsBuilder.fromHttpUrl(baseUrl + applicationExist)
                .queryParam("applicationName", applicationName)
                .queryParam("applicationId", "").toUriString();
        ResponseEntity<Boolean> exchange = restTemplate.exchange(uriString, HttpMethod.GET, entity, Boolean.class, httpParams);
        return exchange.getBody();
    }

    @Override
    public SastFileUploadResp uploadFile(File multipartFile) {
        HttpHeaders httpHeaders = getHeaders();
        MultiValueMap<String, Object> entityMap = new LinkedMultiValueMap<>();
        httpHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);
        FileSystemResource fileSystemResource = new FileSystemResource(multipartFile);
        entityMap.add("multipartFile",fileSystemResource);
        HttpEntity<MultiValueMap<String, Object>> httpEntity = new HttpEntity<>(entityMap,httpHeaders);
        ResponseEntity<String> exchange = restTemplate.exchange(baseUrl+fileUpload, HttpMethod.POST, httpEntity, String.class, new HashMap<>());
        return JSON.parseObject(exchange.getBody(), SastFileUploadResp.class);
    }

    @Override
    public SastIdentifiedLanguageTaskResp createIdentifiedLanguageTask(SastIdentifiedLanguageTaskReq req) {
        HttpHeaders httpHeaders = getHeaders();
        Map<String, List<String>> entityMap = new HashMap<>();
        entityMap.put("fileIds",req.getFileIds());
        HttpEntity<Map<String, List<String>>> entity = new HttpEntity<>(entityMap,httpHeaders);
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        ResponseEntity<SastIdentifiedLanguageTaskResp> exchange = restTemplate.exchange(baseUrl+createIdentifiedLanguageTask, HttpMethod.POST,entity,SastIdentifiedLanguageTaskResp.class,new HashMap<>());
        return exchange.getBody();
    }

    public SastIdentifiedLanguageTaskResp getIdentifiedLanguageTaskStatus(String taskId){
        HttpHeaders httpHeaders = getHeaders();
        Map<String, String> entityMap = new HashMap<>();
//        entityMap.put("taskId",taskId);
        HttpEntity<Map<String, String>> entity = new HttpEntity<>(entityMap,httpHeaders);
//        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        String uriString = UriComponentsBuilder.fromHttpUrl(baseUrl + getIdentifiedLanguageTaskStatus).queryParam("taskId", taskId).toUriString();
        System.out.println(uriString);
        ResponseEntity<SastIdentifiedLanguageTaskResp> exchange = restTemplate.exchange(uriString, HttpMethod.GET,entity,SastIdentifiedLanguageTaskResp.class,new HashMap<>());
        return exchange.getBody();
    }
    public List<SastEngineConfigResp> engineConfig(List<String> languageIdList){
        HttpHeaders httpHeaders = getHeaders();
        Map<String, String> entityMap = new HashMap<>();
        HttpEntity<Map<String, String>> entity = new HttpEntity<>(entityMap,httpHeaders);
        String uriString = UriComponentsBuilder.fromHttpUrl(baseUrl + engineConfig).queryParam("languageIds", StringUtils.joinWith(",", languageIdList.toArray())).toUriString();

        // 别问我为啥要解码。坑爹的不支持Uri encode识别
        String decode = URLDecoder.decode(uriString, StandardCharsets.UTF_8);
        System.out.println(decode);
        ResponseEntity<String> exchange = restTemplate.exchange(decode, HttpMethod.GET,entity, String.class,new HashMap<>());
        return JSON.parseArray(exchange.getBody(), SastEngineConfigResp.class);
    }
    public SastDetectionConfigResp detectionConfig(String languageId){
        HttpHeaders httpHeaders = getHeaders();
        Map<String, String> entityMap = new HashMap<>();
//        entityMap.put("taskId",taskId);
        HttpEntity<Map<String, String>> entity = new HttpEntity<>(entityMap,httpHeaders);
        String uriString = UriComponentsBuilder.fromHttpUrl(baseUrl + detectionConfig).queryParam("languageId", languageId).toUriString();
        System.out.println(uriString);
        ResponseEntity<SastDetectionConfigResp> exchange = restTemplate.exchange(uriString, HttpMethod.GET,entity,SastDetectionConfigResp.class,new HashMap<>());
        return exchange.getBody();
    }

    private HttpHeaders getHeaders(){
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set(TOKEN_HEADER_KEY,TOKEN_PREFIX+getToken());
        return httpHeaders;
    }
}
