package cd.casic.server;

import cd.casic.ci.process.dto.req.sast.SastIdentifiedLanguageTaskReq;
import cd.casic.ci.process.dto.resp.sast.SastDetectionConfigResp;
import cd.casic.ci.process.dto.resp.sast.SastEngineConfigResp;
import cd.casic.ci.process.dto.resp.sast.SastIdentifiedLanguageTaskResp;
import cd.casic.ci.process.process.service.sast.SastService;
import cd.casic.ci.process.properties.SastProperties;
import cd.casic.framework.redis.core.RedisTemplateUtils;
import com.alibaba.fastjson.JSON;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.test.context.ActiveProfiles;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;

@SpringBootTest(classes = {OpsServerApplication.class})
@ActiveProfiles("local")
@Slf4j
public class SastTest {
    @Resource
    SastProperties sastProperties;
    @Resource
    SastService sastService;
    @Resource
    ThreadPoolTaskExecutor workerExecutor;
    @Resource
    RedisTemplateUtils redisTemplateUtils;
    @Test
    public void properties(){
        System.out.println(JSON.toJSONString(sastProperties));
    }
    @Test
    public void getToken(){
        String token = sastService.getToken();
        System.out.println(token);
    }
    @Test
    public void getTokenThread(){

        CyclicBarrier cyclicBarrier = new CyclicBarrier(2);
        String token1 = "";
        String token2 = "";
        redisTemplateUtils.del("REDIS_SAST_TOKEN_KEY");
        workerExecutor.submit(()->{
            try {
                cyclicBarrier.await();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            } catch (BrokenBarrierException e) {
                throw new RuntimeException(e);
            }
            log.info(sastService.getToken());
        });
        workerExecutor.submit(()->{
            try {
                cyclicBarrier.await();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            } catch (BrokenBarrierException e) {
                throw new RuntimeException(e);
            }
            log.info(sastService.getToken());
        });
    }
    @Test
    public void applicationExist(){
        Boolean b = sastService.applicationExist("测试");
        System.out.println(b);
    }
    @Test
    public void fileUpload(){
        File file = new File("C:\\Users\\admin\\Desktop\\新建文件夹\\Java.zip");
        sastService.uploadFile(file);
    }

    @Test
    public void createIdentify(){
        SastIdentifiedLanguageTaskReq req = new SastIdentifiedLanguageTaskReq();
        List<String> fileIds = new ArrayList<>();

        fileIds.add("ed49de43-e2e9-402e-ab10-c6f00d434f43");
        req.setFileIds(fileIds);
        SastIdentifiedLanguageTaskResp resp = sastService.createIdentifiedLanguageTask(req);
        System.out.println(resp);
    }

    @Test
    public void getIdentify(){
        SastIdentifiedLanguageTaskResp resp = sastService.getIdentifiedLanguageTaskStatus("ab8c8ff1-7c07-4787-9107-6624ba0ede02");
        System.out.println(JSON.toJSONString(resp));
    }
    @Test
    public void getEngineConfig(){
        List<String> languageIdList= new ArrayList<>();
        languageIdList.add("17417899-2c50-11ee-a171-e2d048d4b7ce");
        languageIdList.add("17417989-2c50-11ee-a171-e2d048d4b7ce");
        languageIdList.add("17417992-2c50-11ee-a171-e2d048d4b7ce");
        List<SastEngineConfigResp> resp = sastService.engineConfig(languageIdList);
        System.out.println(resp);
    }
    @Test
    public void detectionConfig(){
        SastDetectionConfigResp resp = sastService.detectionConfig("17417989-2c50-11ee-a171-e2d048d4b7ce");
        System.out.println(resp);
    }
}
